from collections import defaultdict
from typing import (
    Dict,
    Optional,
    Protocol,
    Literal
)
from urllib.parse import (
    urljoin,
    urlparse,
)

import requests

Method = Literal['GET', 'HEAD', 'POST', 'PUT', 'DELETE', 'PATCH']


class WireProtocol(Protocol):
    def get(
        self, path: str, query: Optional[Dict[str, str]] = None,
    ) -> str:
        ...


class HTTPWireProtocol(WireProtocol):
    def __init__(
        self,
        url: str,
        headers: Dict[str, str] = None,
        adapter: requests.adapters.BaseAdapter = None,
    ):
        self.url = url
        self.headers = defaultdict(str, headers or {})
        self.session = requests.Session()
        if adapter is not None:
            self.session.mount(f"{urlparse(self.url).scheme}://", adapter)

    def __str__(self):
        return f"<{type(self).__name__} {self.url} object at {hex(id(self))}>"

    def __repr__(self):
        return (
            f"{self.__module__}.{type(self).__name__}("
            f"{self.url!r}, "
            f"headers={dict(self.headers)!r}"
            ")"
        )

    def url_for(self, path: str) -> str:
        return urljoin(self.url, path)

    def get(
        self, path: str, query: Optional[Dict[str, str]] = None,
    ) -> str:
        return self.request('GET', path, query)

    def request(
        self,
        method: Method,
        path: str,
        params: Optional[Dict[str, str]] = None,
    ) -> str:
        resp = self._raw_request(method, path, params=params)
        resp.raise_for_status()
        return resp.text

    def _raw_request(
        self, method: Method, path: str, params: Optional[Dict[str, str]] = None,
    ) -> requests.Response:
        return self.session.request(
            method, self.url_for(path), params=params, headers=self.headers,
        )
