import json
from typing import List, Optional, Union

from pydantic import BaseModel

from ..api import Resource
from ..transport import WireProtocol
from ..utils import quote_path_arg
from .persons import BasePerson


class BadgePersonInfo(BasePerson):
    # this is very similar to a person lookup
    username: Optional[str]
    badge_id: int
    badge_type: int
    badge_key: int

    class Config:
        fields = {
            "username": {"alias": "BRUKERNAVN"},
            "badge_id": {"alias": "BADGEID"},
            "badge_type": {"alias": "BADGETYPE"},
            "badge_key": {"alias": "BADGEKEY"},
        }


class NotFoundResponse(BaseModel):
    none_key: str

    class Config:
        fields = {"none_key": "NoneKey"}


class Badge(BaseModel):
    id: int
    key: int
    status: int
    type: int
    serial_number: str
    username: str
    system_number: int

    class Config:
        fields = {
            "id": {"alias": "ID"},
            "key": {"alias": "BADGEKEY"},
            "status": {"alias": "STATUS"},
            "type": {"alias": "TYPE"},
            "serial_number": {"alias": "SERIAL_NUMBER"},
            "username": {"alias": "BRUKERNAVN"},
            "system_number": {"alias": "SYSTEMNR"},
        }


class Badges(Resource):
    def __init__(self, transport: WireProtocol):
        super().__init__(transport)

    def get_badge_owner(
        self, badge_key: Union[str, int]
    ) -> Optional[Union[BadgePersonInfo, NotFoundResponse]]:
        """
        Look up a badge and get information about its owner.
        """
        url = "badges/{badge_key}&format={format}".format(
            badge_key=quote_path_arg(badge_key), format="json"
        )
        raw_result = self.transport.get(url)
        result = json.loads(raw_result)
        if result is None:
            return None
        try:
            return BadgePersonInfo.parse_obj(result)
        except Exception as exc:
            try:
                return NotFoundResponse.parse_obj(result)
            except Exception:
                raise exc

    def list_badges_for_username(self, username: str) -> List[Badge]:
        """
        List badges associated with a username.

        The endpoint used was originally made for the printing system 'Safecom',
        but at this point it's reused for other purposes.
        """
        # this endpoint expects query parameters in a non-standard format, i.e. no '?'
        url = "safecombadge/{username}&format={format}".format(
            username=quote_path_arg(username),
            format="json",
        )
        raw_result = self.transport.get(url)
        result = json.loads(raw_result)

        class SafecomBadgeList(BaseModel):
            results: List[Badge]

        wrapped = SafecomBadgeList.parse_obj(result)
        return wrapped.results
