import json
from typing import Optional, Union

from pydantic import BaseModel

from ..api import Resource
from ..transport import WireProtocol
from ..utils import quote_path_arg


class BasePerson(BaseModel):
    first_name: str
    last_name: str
    internal_employee_number: int
    employee_number: Optional[int]
    fs_person_id: Optional[int]
    cerebrum_person_id: Optional[str]
    # udfemp_id: int

    class Config:
        fields = {
            "first_name": {"alias": "FIRSTNAME"},
            "last_name": {"alias": "LASTNAME"},
            "employee_number": {"alias": "ANSATTNR"},
            "internal_employee_number": {"alias": "EMPID"},
            "fs_person_id": {"alias": "FSNUMMER"},
            "cerebrum_person_id": {"alias": "CEREBRUM_ID"},
            # "udfemp_id": {"alias": "UDFEMP_ID"},
        }


class PersonSearchResult(BasePerson):
    person_id: str
    ou_code: Optional[str]
    place_of_work: str
    student_id: Optional[int]
    passport_id: Optional[str]

    class Config:
        fields = {
            "person_id": {"alias": "PERSONID"},
            "ou_code": {"alias": "STEDKODE"},
            "place_of_work": {"alias": "ARBEIDSSTED"},
            "student_id": {"alias": "STUDENTNUMMER"},
            "passport_id": {"alias": "PASSNR"},
        }


class Person(BasePerson):
    username: Optional[str]
    borrower_id: Optional[int]
    badge_type: Optional[int]
    badge_key: Optional[int]

    class Config:
        fields = {
            "username": {"alias": "BRUKERNAVN"},
            "borrower_id": {"alias": "LAANENUMMER"},
            "badge_type": {"alias": "BADGETYPE"},
            "badge_key": {"alias": "BADGEKEY"},
        }


class Persons(Resource):
    def __init__(self, transport: WireProtocol):
        super().__init__(transport)

    def get(self, person_id: Union[str, int]) -> Optional[Person]:
        """
        Get a person by the internal Onguard person ID.

        The internal person ID is called 'EMPID',
        or internal_employee_number in our models.
        """
        url = "persons/{person_id}&format={format}".format(
            person_id=quote_path_arg(person_id), format="json"
        )
        raw_result = self.transport.get(url)
        result = json.loads(raw_result)
        if result is None:
            return None
        return Person.parse_obj(result)

    def find(
        self, external_id: Union[str, int], id_type: str
    ) -> Optional[PersonSearchResult]:
        """
        Search for a person by external ID.

        Known ID types:
          - ansattnummer
          - personnummer
          - cerebrum
          - studentnummer
          - passnummer
          - fsnummer
          - programkode
        """
        # this endpoint expects query parameters in a non-standard format, i.e. no '?'
        url = "persons/external-id={external_id}&id_type={id_type}&format={format}".format(
            external_id=quote_path_arg(external_id),
            id_type=quote_path_arg(id_type),
            format="json",
        )
        raw_result = self.transport.get(url)
        result = json.loads(raw_result)
        if result is None:
            return None
        return PersonSearchResult.parse_obj(result)
